const ID_NO_PERSON = null;

export default {
    created() {
        this.ID_NO_PERSON = ID_NO_PERSON;
    },
    methods: {
        removePersonalNote(personID) {
            if (personID === ID_NO_PERSON) {
                return
            }
            console.log("removing personal note of person", personID);
            this.personalNotes.forEach(item => {
                if (item.student.id === personID) {
                    item.tardiness = 0;
                    item.absent = false;
                    item.excused = false;
                    item.excuse_type = null;
                    item.extra_marks = [];
                }
            });

        },
        editPersonalNote(personID) {
            console.log("editing personal note of person", personID);
            this.editedPersonID = personID;
            this.updatePersonalNote();
            this.dialog = true;
        },
        updatePersonalNote() {
            let personalNote = this.personalNoteByStudentID(this.editedPersonID);
            this.editedTardiness = personalNote.tardiness || 0;
            this.editedAbsent = personalNote.absent || false;
            this.editedExcused = personalNote.excused || false;
            this.editedExcuseType = personalNote.excuse_type || null;
            this.editedExtraMarks = personalNote.extra_marks || [];
        },
        personalNoteByStudentID(studentID) {
            if (this.editedPersonID === ID_NO_PERSON) {
                return {};
            }
            return this.personalNotes.filter(item => item.student.id === studentID)[0] || {};
        },
        savePersonalNote() {
            if (this.editedPersonID === ID_NO_PERSON) {
                return
            }

            // Loop through all personal notes and update the ones that match the editedPersonID
            this.personalNotes.forEach(item => {
                if (item.student.id === this.editedPersonID) {
                    item.tardiness = this.editedTardiness;
                    item.absent = this.editedAbsent;
                    item.excused = this.editedExcused;
                    item.excuse_type = this.editedExcuseType;
                    item.extra_marks = this.editedExtraMarks;
                }
            });
        },
        cancelDialog() {
            this.dialog = false;
            this.editedPersonID = ID_NO_PERSON;
        },
        saveDialog() {
            this.savePersonalNote();
            this.dialog = false;
            this.editedPersonID = ID_NO_PERSON;
        },
        personalNoteString(personalNote) {
            let personalNoteString = "";
            if (personalNote.tardiness > 0) {
                personalNoteString += personalNote.tardiness + " min. ";
            }
            if (personalNote.absent) {
                personalNoteString += "abwesend ";
            }
            if (personalNote.excused) {
                personalNoteString += "entschuldigt ";
            }
            if (personalNote.excuse_type) {
                personalNoteString += personalNote.excuse_type.name;
            }
            if (personalNote.extra_marks.length > 0) {
                personalNoteString += " (";
                personalNote.extra_marks.forEach(item => {
                    personalNoteString += item.name + ", ";
                });
                personalNoteString = personalNoteString.substring(0, personalNoteString.length - 2);
                personalNoteString += ")";
            }
            return personalNoteString;
        }
    },
    props: ["personalNotes", "groups", "excuseTypes", "extraMarks"],
    name: "personal-notes",
    data: () => {
        return {
            dialog: false,
            // Absent versp. exc. type hw note
            editPersonalNoteId: null,
            editedPersonID: ID_NO_PERSON,
            editedTardiness: 0,
            editedAbsent: false,
            editedExcused: false,
            editedExcuseType: null,
            editedExtraMarks: [],
        }
    },
    computed: {
        persons() {
            // go through each group and get the students
            // use the group names as headers for the v-select

            return this.groups.map(
                group => {
                    return [
                        {header: group.name, id: group.short_name},
                        group.members
                    ]
                }
            ).flat(2);
        }
    },
    template: `
    <v-dialog
      v-model="dialog"
      max-width="600px"
      @click:outside="cancelDialog"
    >
      <template v-slot:activator="{ on, attrs }">
        <div>
          <template v-for="personal_note in personalNotes">
            <v-chip class="ma-1" close @click="editPersonalNote(personal_note.student.id)"
              @click:close="removePersonalNote(personal_note.student.id)" v-if="personalNoteString(personal_note)">
              {{ personal_note.student.full_name }}: {{ personalNoteString(personal_note) }}
            </v-chip>
          </template>
        </div>
        <v-btn
          class="ma-1"
          color="primary"
          icon
          outlined
          v-bind="attrs"
          v-on="on"
        >
          <v-icon>
            mdi-plus
          </v-icon>
        </v-btn>
      </template>
      <v-card>
        <v-card-title>
          <span class="text-h5">Personal Note</span>
        </v-card-title>
        <v-card-text>
          <v-container>
            <v-select
              item-text="full_name"
              item-value="id"
              :items="persons"
              label="Student"
              v-model="editedPersonID"
              @input="updatePersonalNote"
            ></v-select>
            <v-text-field
              label="Tardiness"
              suffix="min" type="number"
              min="0"
              :disabled="editedPersonID === ID_NO_PERSON"
              v-model="editedTardiness"
            ></v-text-field>
            <v-checkbox
              label="Absent"
              v-model="editedAbsent"
              :disabled="editedPersonID === ID_NO_PERSON"
              @change="editedExcused = false; editedExcuseType = null"
            ></v-checkbox>
            <v-checkbox
              label="Excused"
              v-model="editedExcused"
              :disabled="editedPersonID === ID_NO_PERSON || !editedAbsent"
              @change="editedExcuseType = null"
            ></v-checkbox>
            <v-select
              label="Excuse Type"
              v-model="editedExcuseType"
              :items="excuseTypes"
              item-text="name"
              item-value="short_name"
              :disabled="editedPersonID === ID_NO_PERSON || !editedAbsent || !editedExcused"
            ></v-select>
            <!-- FIXME: get extra mark itself as value -->
            <v-select
              label="Extra Marks"
              v-model="editedExtraMarks"
              :items="extraMarks"
              item-text="name"
              item-value="valueOf"
              :disabled="editedPersonID === ID_NO_PERSON"
              multiple
              chips
            ></v-select>
          </v-container>
<!--          <small>*indicates required field</small>-->
        </v-card-text>
        <v-card-actions>
          <v-spacer></v-spacer>
          <v-btn
            color="error"
            outlined
            @click="cancelDialog"
          >
            Cancel
          </v-btn>
          <v-btn
            color="success"
            @click="saveDialog"
            :disabled="editedPersonID === ID_NO_PERSON"
          >
            Save
          </v-btn>
        </v-card-actions>
      </v-card>
    </v-dialog>
    `
}